// EDock (C)opyright 2007 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <gui/window.h>		// Due to GetWindow()
#include <gui/font.h>		// Due to font_height
#include <appserver/protocol.h>
#include <gui/menu.h>
#include "windowbutton.h"

using namespace os;

///////////////////////////////////////////////////////////////////////////////
//
// P R I V A T E
//
///////////////////////////////////////////////////////////////////////////////

class WindowButton :: _Private
{
public:
	_Private()
	{
		m_zTitle = "";
		m_zID = "";
		m_nHandle = -1;
		m_bMinimized = false;
		m_bVisible = true;
		m_hMsgPort = -1;
		m_nDesktopMask = 0;

		m_WindowMenu = NULL;
		m_Bitmap = NULL;
		m_MinimizedBitmap = NULL;
	}

	enum WindowButtonMessage { MSG_MINIMIZE, MSG_UNMAXIMAZE, MSG_TOP, MSG_MOVE, MSG_RESIZE, MSG_CLOSE };
	Menu* m_WindowMenu;
	BitmapImage* m_Bitmap;
	BitmapImage* m_MinimizedBitmap;
	String m_zID;
	String m_zTitle;
	bool m_bVisible;
	bool m_bMinimized;
	int64 m_nHandle;
	port_id m_hMsgPort;
	uint32 m_nDesktopMask;
};

///////////////////////////////////////////////////////////////////////////////
//
// T H E   C L A S S
//
///////////////////////////////////////////////////////////////////////////////
WindowButton :: WindowButton( Rect cFrame, const String& cName, const String& cLabel, Message* pcMessage, uint32 nResizeMask, uint32 nFlags ) : Button( cFrame, cName, cLabel, pcMessage, nResizeMask, nFlags )
{	
	// Create the private class
	m = new _Private();

}

WindowButton :: ~WindowButton()
{
	delete m;
}

WindowButton* WindowButton ::Duplicate()
{
	WindowButton* tmp = new WindowButton( GetFrame(), GetName(), GetLabel(), new Message( *GetMessage() ) );

	// Make copy of bitmap
	if( GetImage() != NULL && GetImage()->LockBitmap() != NULL )
	{
		BitmapImage* img = new BitmapImage();
		img->SetBitmapData( GetImage()->LockBitmap()->LockRaster(), IPoint( 24, 24 ), CS_RGBA32);
		tmp->SetImage( img );
	}
	tmp->SetMsgPort( GetMsgPort() );
	tmp->SetID( GetID() );
	tmp->SetTitle( GetTitle() );	
	tmp->SetHandle( GetHandle() );
	tmp->SetVisible( GetVisible( GetDesktopMask() ) );
	tmp->SetMinimized( GetMinimized());
	tmp->SetDesktopMask( GetDesktopMask() );
	tmp->SetTarget( GetMessenger() );
	return tmp;
}

BitmapImage* WindowButton :: GetImage()
{
	return m->m_Bitmap;
}

void WindowButton :: SetImage( BitmapImage * pcBitmap )
{
	if( m->m_Bitmap )
		delete m->m_Bitmap;

	m->m_Bitmap = pcBitmap;

	// Create a minimized image
	m->m_MinimizedBitmap = new os::BitmapImage();
	os::Bitmap* tmp = m->m_Bitmap->LockBitmap();
	m->m_MinimizedBitmap->SetBitmapData( tmp->LockRaster(), os::IPoint( 24, 24 ), os::CS_RGB32 );
	m->m_MinimizedBitmap->ApplyFilter( os::Image::F_GRAY );
	m->m_Bitmap->UnlockBitmap();
}

port_id WindowButton :: GetMsgPort()
{ 
	return( m->m_hMsgPort ); 
}

void WindowButton :: SetMsgPort( port_id hPort ) 
{ 
	m->m_hMsgPort = hPort; 
}

os::String WindowButton :: GetID() 
{ 
	return( m->m_zID ); 
}

void WindowButton :: SetID( os::String zID ) 
{ 
	m->m_zID = zID; 
}
	
os::String WindowButton :: GetTitle() 
{ 
	return( m->m_zTitle ); 
}

void WindowButton :: SetTitle( os::String zTitle ) 
{ 
	m->m_zTitle = zTitle; 
	SetLabel( zTitle );
}
	
int64 WindowButton :: GetHandle()
{
	return( m->m_nHandle );
}

void WindowButton :: SetHandle( int64 nHandle ) 
{ 
	m->m_nHandle = nHandle; 
}

bool WindowButton :: GetVisible( uint32 nDesktopMask )
{
	return( ( m->m_bVisible || m->m_bMinimized ) && ( nDesktopMask & m->m_nDesktopMask ) );
}

void WindowButton :: SetVisible( bool bVisible )
{
	m->m_bVisible = bVisible; 
}
	
bool WindowButton :: GetMinimized()
{
	return( m->m_bMinimized );
}

void WindowButton :: SetMinimized( bool bMinimized )
{ 
	m->m_bMinimized = bMinimized; 
}

uint32 WindowButton :: GetDesktopMask()
{
	return( m->m_nDesktopMask );
}

void WindowButton :: SetDesktopMask( uint32 nMask ) 
{ 
	m->m_nDesktopMask = nMask; 
}

void WindowButton :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{	
		case _Private::MSG_MINIMIZE:
		{
			Messenger cMessenger( GetMsgPort() );
			cMessenger.SendMessage( os::WR_MINIMIZE );
			break;
		}
		case _Private::MSG_UNMAXIMAZE:
			break;
		case _Private::MSG_TOP:
		{
			Messenger cMessenger( GetMsgPort() );
			cMessenger.SendMessage( os::WR_ACTIVATE );
			break;
		}
		case _Private::MSG_MOVE:
		case _Private::MSG_RESIZE:
			break;
		case _Private::MSG_CLOSE:
		{
			Messenger cMessenger( GetMsgPort() );
			cMessenger.SendMessage( os::M_QUIT );
			break;
		}
		default:
			View::HandleMessage( pcMessage );
			break;
	}
}

void WindowButton :: MouseDown( const Point& cPosition, uint32 nButtons )
{
	if( nButtons != 1 )
	{
		// Create popup menu
		if( m->m_WindowMenu == NULL )
		{
			m->m_WindowMenu = new os::Menu( os::Rect(), "windowmenu", os::ITEMS_IN_COLUMN );
			m->m_WindowMenu->AddItem( "Minimize", new os::Message( _Private::MSG_MINIMIZE ) );
		//	m->m_WindowMenu->AddItem( "Unmaximaze", new os::Message( _Private::MSG_UNMAXIMAZE ) );
			m->m_WindowMenu->AddItem( "Top", new os::Message( _Private::MSG_TOP ) );
		//	m->m_WindowMenu->AddItem( "Move", new os::Message( _Private::MSG_MOVE ) );
		//	m->m_WindowMenu->AddItem( "Resize", new os::Message( _Private::MSG_RESIZE ) );
			m->m_WindowMenu->AddItem( new MenuSeparator() );
			m->m_WindowMenu->AddItem( "Close", new os::Message( _Private::MSG_CLOSE ) );

			m->m_WindowMenu->SetTargetForItems( this );
		}

		m->m_WindowMenu->Open( ConvertToScreen( cPosition ) );	
	}
	else
		Button::MouseDown( cPosition, nButtons );
}

Point WindowButton :: GetPreferredSize( bool bLargest ) const
{
	if( bLargest )
	{
		return Point( COORD_MAX, COORD_MAX );
	}

	return Point( 20, 40 );
}

void WindowButton :: Paint( const Rect& cUpdateRect )
{
	Rect cBounds = GetBounds();

	SetEraseColor( get_default_color( COL_NORMAL ) );

	uint32 nFrameFlags = ( GetValue().AsBool(  ) && IsEnabled(  ) )? FRAME_RECESSED : FRAME_RAISED;

	if( IsEnabled() && ( HasFocus() || ( GetWindow(  )->GetDefaultButton(  ) == this ) ) )
	{
		if( HasFocus() ) {
			SetFgColor( get_default_color( COL_FOCUS ) );
		} else {
			SetFgColor( 0, 0, 0 );
		}
		DrawLine( Point( cBounds.left, cBounds.top ), Point( cBounds.right, cBounds.top ) );
		DrawLine( Point( cBounds.right, cBounds.bottom ) );
		DrawLine( Point( cBounds.left, cBounds.bottom ) );
		DrawLine( Point( cBounds.left, cBounds.top ) );
		cBounds.Resize( 1, 1, -1, -1 );
	}

	DrawFrame( cBounds, nFrameFlags );

	if( IsEnabled() )
	{
		SetFgColor( 0, 0, 0 );
	}
	else
	{
		SetFgColor( 255, 255, 255 );
	}
	SetBgColor( get_default_color( COL_NORMAL ) );

	if( GetValue().AsBool(  ) )
	{
		cBounds.MoveTo( 1, 1 );
	}

	int align = DTF_ALIGN_CENTER | DTF_UNDERLINES;

	if( m->m_Bitmap )
	{
		SetDrawingMode( DM_BLEND );	//DM_BLEND
		Point pos = Point( cBounds.left + 3.0f, ( cBounds.bottom - cBounds.top ) / 2.0f + cBounds.top - m->m_Bitmap->GetSize().y / 2.0f );

		if( GetMinimized() )
			m->m_MinimizedBitmap->Draw( pos , this );
		else
			m->m_Bitmap->Draw( pos , this );

		SetDrawingMode( DM_OVER );	//sets the mode to dm_over, so that hovering works completely

		cBounds.left += 9.0f + m->m_Bitmap->GetSize().x;

		align = DTF_ALIGN_LEFT | DTF_UNDERLINES;
	}

	cBounds.right -= 2.0f;

	// Set textcolour
	if( GetMinimized() )
		SetFgColor( 100, 100, 100 );

	DrawText( cBounds, GetLabel(), align );
	if( IsEnabled() == false )
	{
		SetFgColor( 100, 100, 100 );
		SetDrawingMode( DM_OVER );
		cBounds.MoveTo( -1, -1 );
		DrawText( cBounds, GetLabel(), align );
		SetDrawingMode( DM_COPY );
	}
}
