// EDock (C)opyright 2007 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <vector>

#include <gui/layoutview.h>
#include <appserver/protocol.h>
#include <gui/window.h>
#include "main.h"
#include "windowview.h"
#include "windowbutton.h"


using namespace os;
using namespace std;

///////////////////////////////////////////////////////////////////////////////
//
// P R I V A T E
//
///////////////////////////////////////////////////////////////////////////////

class WindowView :: _Private
{
public:
	_Private()
	{
	
	}

	void _AddDistance()
	{
		HLayoutSpacer* space = new HLayoutSpacer( "space" );
		space->LimitMaxSize( Point( 10, 1 ) );
		m_RootNode->AddChild( space );
	}

	void _CreateNewWindowView()
	{
		// Get Current desktop
		uint32 nDesktopMask = 1 << EDockApp :: GetAppWindow()->GetDesktop();

		m_RootNode = new HLayoutNode( "pcRootNode" );
		m_RootNode->SetBorders( Rect( 1, 1, 1, 1 ) );
		m_RootView->SetRoot( m_RootNode ); // all children will be deleted when we change root

		for( uint i = 0 ; i < m_WindowButtons.size() ; i++ )
		{
			WindowButton* t = m_WindowButtons[ i ];

			if( !t->GetVisible( nDesktopMask ) )
				continue;

			_AddDistance();

			// Create copy
			WindowButton* tmp = t->Duplicate();
			m_RootNode->AddChild( tmp );
		}
		m_RootView->InvalidateLayout();
	}

	void _ActivateWindow( Message* pcMessage )
	{
		int64 msg_port;
		if( pcMessage->FindInt64( "msg_port", &msg_port ) == EOK )
		{
			Messenger cMessenger( msg_port );
			cMessenger.SendMessage( os::WR_ACTIVATE );
		}
	}

	enum WindowViewMessage { MSG_BUTTON };

	vector< WindowButton* >	m_WindowButtons;
	vector< WindowButton* >	m_VisibleButtons;

	// Widgets
	LayoutView*	m_RootView;
	LayoutNode*	m_RootNode;
};

///////////////////////////////////////////////////////////////////////////////
//
// T H E   C L A S S
//
///////////////////////////////////////////////////////////////////////////////
WindowView :: WindowView() : View( os::Rect(), "window_view" )
{	
	// Create the private class
	m = new _Private();

	// Create widgets
	m->m_RootNode = new HLayoutNode( "pcRootNode" );
	m->m_RootNode->SetBorders( Rect( 1, 1, 1, 1 ) );
	m->m_RootView = new LayoutView( GetBounds(), "pcRootView" );
	m->m_RootView->SetRoot( m->m_RootNode );
	AddChild( m->m_RootView );
}

WindowView :: ~WindowView()
{
	delete m;
}

void WindowView :: HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{	
		case _Private::MSG_BUTTON:
			m->_ActivateWindow( pcMessage );
			break;
		default:
			View::HandleMessage( pcMessage );
			break;
	}
}

Point WindowView ::  GetPreferredSize( bool bLargest ) const
{
	// This view wants to always be as large as possible
	return Point( COORD_MAX, COORD_MAX );
}

void WindowView :: UpdateWindowView( Message* pcMessage )
{
	String zID;
	bool bDummy;	
	int64 hMsgPort;
	
	pcMessage->FindString( "id", &zID );
	
	// Check if the window shall be removed
	if( pcMessage->FindBool( "event_unregistered", &bDummy ) == 0 )
	{
		// Remove window
		for( uint i = 0; i < m->m_WindowButtons.size(); i++ )
		{
			if( m->m_WindowButtons[i]->GetID() == zID )
			{
				delete( m->m_WindowButtons[i] );
				m->m_WindowButtons.erase( m->m_WindowButtons.begin() + i );
				m->_CreateNewWindowView();
				return;
			}
		}
		printf( "Error: Cannot remove entry %s\n", zID.c_str() );
		return;
	}

	// Add window	
	WindowButton* button = NULL;

	String zTitle;
	pcMessage->FindString( "title", &zTitle );	
	String zApp;
	pcMessage->FindString( "application", &zApp );
	
	// New window
	if( pcMessage->FindBool( "event_registered", &bDummy ) == 0 )
	{
		// Add new window
		
		pcMessage->FindInt64( "target", &hMsgPort );
		
		
		Message* msg = new Message( _Private::MSG_BUTTON );
		msg->AddString( "id", zID );
		msg->AddInt64( "msg_port", hMsgPort );

		button = new WindowButton( Rect(), "", zTitle, msg );
		button->SetTarget( this );
		button->SetMsgPort( hMsgPort );
		m->m_WindowButtons.push_back( button );		
	}

	// Changed window
	else
	{
		// Change already present window
		uint i = 0;
		for( i = 0; i < m->m_WindowButtons.size(); i++ )
		{
			if( m->m_WindowButtons[i]->GetID() == zID )
			{
				button =  m->m_WindowButtons[i];
				break;
			}
		}
		if( i == m->m_WindowButtons.size() )
		{
			printf( "Error: Cannot find entry %s\n", zID.c_str() );
			return;
		}
	}

	// Set icon
	int64 nHandle;
	if( pcMessage->FindInt64( "icon_handle", &nHandle ) == 0 )
	{
		/* Map icon */
		bool bIconChanged = false;
		pcMessage->FindBool( "icon_changed", &bIconChanged );
		
//		if( nHandle != pcIcon->GetHandle() || bIconChanged )
		{
			Bitmap* pcBitmap = NULL;
			try
			{
				pcBitmap = new os::Bitmap( nHandle );
				BitmapImage* image = new BitmapImage();

				os::Point cSize = pcBitmap->GetBounds().Size();

				// Copy from bitmap to the bitmapimage
				image->SetBitmapData( pcBitmap->LockRaster(), os::IPoint( (int)cSize.x + 1, (int)cSize.y + 1 ), os::CS_RGB32 );	

				// Resize icon if needed
				if( image->GetSize() != os::Point( 24, 24 ) )
					image->SetSize( os::Point( 24, 24 ) );

				delete pcBitmap;
				button->SetImage( image );

			} catch( ... )
			{
			}
		}
	}

	button->SetTitle( zTitle );
	button->SetID( zID );

	int64 nDesktopMask;
	if( pcMessage->FindInt64( "desktop_mask", &nDesktopMask ) == 0 )
		button->SetDesktopMask( nDesktopMask );
	bool bMinimized;
	if( pcMessage->FindBool( "minimized", &bMinimized ) == 0 )
		button->SetMinimized( bMinimized );

	m->_CreateNewWindowView();
}

void WindowView :: UpdateWindowView()
{
	m->_CreateNewWindowView();
}


